@@ -43,16 +43,22 @@ sub parse
     {
       $doc->{system}->{load_modules}->{module} = [ ];
     }# end if()
-    
-    $doc->{system}->{env_vars} ||= [ ];
+
+    $doc->{system}->{env_vars} ||= { };
     if( $doc->{system}->{env_vars}->{var} )
     {
-      $doc->{system}->{env_vars} = [ $doc->{system}->{env_vars}->{var} ]
+      $doc->{system}->{env_vars}->{var} = [ delete($doc->{system}->{env_vars}->{var}) ]
         unless ref($doc->{system}->{env_vars}->{var}) eq 'ARRAY';
+      my $ref = delete($doc->{system}->{env_vars}->{var});
+      $doc->{system}->{env_vars}->{var} = [ ];
+      foreach my $item ( @$ref )
+      {
+        push @{$doc->{system}->{env_vars}->{var}}, { $item->{name} => $item->{value} };
+      }# end foreach()
     }
     else
     {
-      $doc->{system}->{env_vars}= [ ];
+      $doc->{system}->{env_vars}->{var} = [ ];
     }# end if()
     
     # Post-processor:
@@ -70,15 +76,20 @@ sub parse
   
   WEB: {
     $doc->{web}->{settings} ||= { };
-    $doc->{web}->{request_filters} ||= { };
-    if( $doc->{web}->{request_filters}->{filter} )
+    if( $doc->{web}->{settings}->{setting} )
     {
-      $doc->{web}->{request_filters}->{filter} = [ $doc->{web}->{request_filters}->{filter} ]
-        unless ref($doc->{web}->{request_filters}->{filter}) eq 'ARRAY';
+      $doc->{web}->{settings}->{setting} = [ delete($doc->{web}->{settings}->{setting}) ]
+        unless ref($doc->{web}->{settings}->{setting}) eq 'ARRAY';
+      my $ref = delete($doc->{web}->{settings}->{setting});
+      $doc->{web}->{settings}->{setting} = [ ];
+      foreach my $item ( @$ref )
+      {
+        push @{$doc->{web}->{settings}->{setting}}, { $item->{name} => $item->{value} };
+      }# end foreach()
     }
     else
     {
-      $doc->{web}->{request_filters}->{filter} = [ ];
+      $doc->{web}->{settings}->{setting} = [ ];
     }# end if()
   };
   
